#ifndef SINKPLUGIN_H
#define SINKPLUGIN_H

#include <QtGui>

class PluginInterface;
class MaemoBarcodeWindow;

// IMPORTANT: For some reason every virtual function must have a "= 0" for the function to be loaded (no const!)
// If anyone knows why, please tell me (dragly)

class PluginAction : public QObject {
    //Q_OBJECT
public:
    PluginAction(const PluginInterface *interface) {this->interface = interface; this->priority=99; this->enabled=false;}
    const PluginInterface* getInterface() {return this->interface;}
    virtual ~PluginAction() {}

    // virtuals (all are optional to make sure the plugin loads even though the developer has made a mistake)
    virtual QString getName() {return "Sink plugin name not defined";} // the name of the sink plugin
    virtual QString getText() {return "Plugin text not defined";} // the text to show in the list of results (for instance: "Found 300 products online. Lowest price: €200").
    virtual QString getInformativeText() {return "";} // text shown below getText
    virtual QString getDataText() {return "";} // text shown to the far right unless an image is defined
    virtual QImage getDataImage() {return QImage();} // the data image
    virtual QImage getImage() {return QImage();} // an image to be returned in the results window or history list (might be a logo or basically anything).
    virtual void clickAction(QWidget *parentWindow) {Q_UNUSED(parentWindow)} // an action to be performed when the plugin is clicked in the results list

    // Not yet implemented
    virtual bool hasResultsWindowWidget() {return false;} // true if the plugin should have a custom widget
    virtual QWidget *resultsWindowWidget() {return new QWidget();} // the custom widget for the results window
    virtual bool hasHistoryListWidget() {return false;} // true if the plugin has a custom widget
    virtual QWidget *historyListWidget() {return new QWidget();} // a custom widget for the history list

    bool operator< (const PluginAction &action1) const {return this->priority<action1.priority;}
    //friend bool operator> (const PluginAction &action1) {return this->priority>action1.priority;}

    const int getPriority() {return this->priority;}
    const bool getEnabled() {return this->enabled;}
    void setPriority(int priority) {this->priority = priority;}
    void setEnabled(bool enabled) {this->enabled = enabled;}

//signals:
//    void isReady();

private:
    const PluginInterface* interface;
    int priority;
    bool enabled;
};

Q_DECLARE_METATYPE(PluginAction *);

#endif // SINKPLUGIN_H
